(*

  (C) 2008-2013 Alexander Myasnikov
  --= RSD implementation =--

  Web: ru-steganography.narod.ru

  e-mail: amsoftware@ya.ru


  USAGE:

  if USERSD then
  begin

  //Entering RSD mode

  EnterRSDMode(PASSWORD, bmp_datasize);

  // RSD ready!

  for cnt := 0 to bmp_datasize - 1 do
  begin
  target := rsd_block[cnt];
  Buf[target] := Buf[target] xor am.PRNG;  // Crypt by stream cipher
  end;

  ExitRSDMode;
  end
  else
  begin
  for cnt := 0 to bmp_datasize - 1 do
  Buf[cnt] := Buf[cnt] xor am.PRNG;
  end;

  // Lets embed!

  for cnt := 0 to bmp_datasize - 1 do
  for i := 0 to 7 do
  BB[cnt * 8 + i] := (Buf[cnt] shr seed[i]) and 1;


*)

unit rsd_bitmap;

interface

procedure EnterRSDMode(password: String; length: integer);
procedure ExitRSDMode;

var
  rsd_block: array of longword;

implementation

uses amprng10;

var
  am: TAMPRNG;

var
  used: array of boolean;

procedure EnterRSDMode(password: String; length: integer);
var
  i: integer;
  ni: integer;
begin

  am := TAMPRNG.Create(password, 64, 768);
  SetLength(rsd_block, length);
  SetLength(used, length);

  for i := 0 to length - 1 do
    rsd_block[i] := $FFFFFFFF;

  for i := 0 to length - 1 do
    used[i] := False;

  for i := 0 to length - 1 do
  begin

    repeat
      ni := am.RandomU32 mod length;
    until used[ni] = False;

    rsd_block[i] := ni;
    used[ni] := true;

  end;

end;

procedure ExitRSDMode;
begin
  am.Free;
  SetLength(rsd_block, 0);
  SetLength(used, 0);
end;

end.
